package me.lensfrex.littlebusters.api.v1.login;

import com.google.gson.Gson;
import com.google.gson.JsonParseException;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.MediaType;
import me.lensfrex.littlebusters.api.v1.beans.requests.LoginRequestBody;
import me.lensfrex.littlebusters.api.v1.beans.responses.LoginResponseData;
import me.lensfrex.littlebusters.api.v1.beans.responses.general.ErrorResponse;
import me.lensfrex.littlebusters.api.v1.beans.responses.general.ResponseBase;
import me.lensfrex.littlebusters.api.v1.pojos.UserInformation;
import me.lensfrex.littlebusters.api.v1.utils.InputChecker;
import me.lensfrex.littlebusters.api.v1.utils.database.User;
import me.lensfrex.littlebusters.api.v1.utils.jwt.JWTManager;
import org.mindrot.jbcrypt.BCrypt;

import java.time.LocalDateTime;

@Path("/login")
public class Login {
    private final static int TOKEN_EXPIRE_DAY = 15;
    private final static Gson gson = new Gson();
    private final static JWTManager jwtManager = new JWTManager();
    User userTools = new User();

    @POST
    @Produces(MediaType.APPLICATION_JSON)
    public String login(String request) {
        LoginRequestBody loginRequestBody;
        try {
            loginRequestBody = gson.fromJson(request, LoginRequestBody.class);
            if (InputChecker.hasInvalidChar(loginRequestBody.getUserName()) ||
                    InputChecker.hasInvisibleChar(loginRequestBody.getPassword())) {
                ErrorResponse errorResponse = new ErrorResponse(100, "请求的用户名或密码非法");

                return gson.toJson(errorResponse);
            }

            UserInformation userDatabaseInformation = userTools.getUser(loginRequestBody.getUserName());

            // todo 这些情况用异常处理会好点吧 分散在代码里面很难看
            // 过会改改
            if (userDatabaseInformation == null) {
                ErrorResponse errorResponse = new ErrorResponse(400, "用户名或密码错误");
                return gson.toJson(errorResponse);
            }

            if (!identifyPassword(loginRequestBody.getPassword(), userDatabaseInformation.password)) {
                ErrorResponse errorResponse = new ErrorResponse(400, "用户名或密码错误");
                return gson.toJson(errorResponse);
            }

            String userToken = jwtManager.createNewJWT("login", loginRequestBody.getUserName(), TOKEN_EXPIRE_DAY);

            LoginResponseData loginResponseData = new LoginResponseData(
                    userDatabaseInformation.uid,
                    userDatabaseInformation.uuid,
                    userToken,
                    LocalDateTime.now().plusDays(TOKEN_EXPIRE_DAY).toString());

            ResponseBase<LoginResponseData> response = new ResponseBase<>(200, "success", loginResponseData);

            return gson.toJson(response);

        } catch (JsonParseException e) {
            System.err.println(request);

            ErrorResponse errorResponse = new ErrorResponse(300, "请求的数据格式不对");
            return gson.toJson(errorResponse);
        } catch (Exception e) {
            System.err.println(request);
            System.err.println(e.getMessage());

            ErrorResponse errorResponse = new ErrorResponse(400, "服务器内部错误，请联系那个背锅的家伙");
            return gson.toJson(errorResponse);
        }
    }

    public boolean identifyPassword(String originPassword, String bcryptPassword) {
        return BCrypt.checkpw(originPassword, bcryptPassword);
    }
}